// ChildView.cpp : implementation of the CChildView class
//

#include "stdafx.h"
#include "HPaint.h"
#include "ChildView.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

bool PointOnCircle(CPoint a, CPoint b, float w, CPoint tst);
bool PointOnLine(CPoint a, CPoint b, float w, CPoint tst);
float DistPointOnLine(CPoint a, CPoint b, CPoint tst);

CPoint click, rotcenter, vecstart;
BOOL movingPt1 = FALSE,
	 movingPt2 = FALSE,
	 movingPt3 = FALSE,
	 rotating = FALSE;

int lin_cur;
int tri_cur;
int quad_cur;
int circ_cur;

static UINT nWidths[5] = { 1, 8, 16, 24, 32 };

static UINT nStyles[3] = { PS_SOLID, PS_DASH, PS_DOT };

const COLORREF CChildView::crColors[8] = {
    RGB (  0,   0,   0),    // Black
    RGB (  0,   0, 255),    // Blue
    RGB (  0, 255,   0),    // Green
    RGB (  0, 255, 255),    // Cyan
    RGB (255,   0,   0),    // Red
    RGB (255,   0, 255),    // Magenta
    RGB (255, 255,   0),    // Yellow
    RGB (255, 255, 255)     // White
};

/////////////////////////////////////////////////////////////////////////////
// CChildView

CChildView::CChildView()
{
	m_nTool  = IDM_TOOL_LINE;
    m_nColor = IDM_COLOR_RED;
    m_nWidth = IDM_WIDTH_MEDIUM;
	m_nStyle = IDM_STYLE_SOLID;

    m_lineArray.SetSize (0, 64);
	m_triArray.SetSize (0, 64);
	m_quadArray.SetSize (0, 64);
	m_circArray.SetSize (0, 64);
	m_nSelectedLine = -1; // line selected
	m_nRubberLine = -1;   // rubberbanding flag

	m_nSelectedTriangle = -1;
	m_nRubberTriangle = -1;

	m_nSelectedQuad = -1;
	m_nRubberQuad = -1;

	m_nSelectedCirc = -1;
	m_nRubberCirc = -1;

	tri_cur = lin_cur = quad_cur = circ_cur = 0;
}

CChildView::~CChildView()
{
    DeleteAllLines ();
}


BEGIN_MESSAGE_MAP(CChildView,CWnd )
	//{{AFX_MSG_MAP(CChildView)
	ON_WM_PAINT()
    ON_COMMAND (IDM_FILE_NEW, OnFileNew)
    ON_COMMAND_RANGE (IDM_WIDTH_VTHIN, IDM_WIDTH_VTHICK, OnWidth)
    ON_COMMAND_RANGE (IDM_COLOR_BLACK, IDM_COLOR_WHITE, OnColor)
	ON_COMMAND_RANGE (IDM_TOOL_PICK, IDM_TOOL_CIRC, OnTool)
	ON_COMMAND_RANGE (IDM_STYLE_SOLID, IDM_STYLE_DOT, OnStyle)
//	ON_COMMAND_RANGE (IDM_VIEW_TOOLBAR, IDM_VIEW_COLORBAR, OnView)
    ON_UPDATE_COMMAND_UI (IDM_FILE_NEW, OnUpdateFileUI)
    ON_UPDATE_COMMAND_UI_RANGE (IDM_WIDTH_VTHIN, IDM_WIDTH_VTHICK,
        OnUpdateWidthUI)
    ON_UPDATE_COMMAND_UI_RANGE (IDM_COLOR_BLACK, IDM_COLOR_WHITE,
        OnUpdateColorUI)
    ON_UPDATE_COMMAND_UI_RANGE (IDM_TOOL_PICK, IDM_TOOL_CIRC,
        OnUpdateToolUI)
    ON_UPDATE_COMMAND_UI_RANGE (IDM_STYLE_SOLID, IDM_STYLE_DOT,
        OnUpdateStyleUI)
    ON_WM_LBUTTONDOWN ()
    ON_WM_MOUSEMOVE ()
    ON_WM_LBUTTONUP ()
    ON_WM_MEASUREITEM ()
    ON_WM_DRAWITEM ()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CChildView message handlers

BOOL CChildView::PreCreateWindow(CREATESTRUCT& cs) 
{
	if (!CWnd::PreCreateWindow(cs))
		return FALSE;

	cs.dwExStyle |= WS_EX_CLIENTEDGE;
	cs.style &= ~WS_BORDER;
	cs.lpszClass = AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, 
		::LoadCursor(NULL, IDC_CROSS), HBRUSH(COLOR_WINDOW+1), NULL);

	return TRUE;
}

void CChildView::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
    int nCount = m_lineArray.GetSize ();

    if (nCount) 
	{
        for (int i=0; i<nCount; i++)
		{
			// draw black squares on the selected object
			if(m_nSelectedLine == i)
			{
				((CLine*) m_lineArray[i])->Draw (&dc);
				if(m_nTool == IDM_TOOL_PICK)
					((CLine*) m_lineArray[i])->DrawEndsB (&dc);
				else
					((CLine*) m_lineArray[i])->DrawEndsBC (&dc);
			}

			if((m_nTool == IDM_TOOL_PICK) ||
				(m_nTool == IDM_TOOL_ERASE))
			{
				// draw white endpoints when line is not selected
				if(m_nSelectedLine != i)
				{
					((CLine*) m_lineArray[i])->Draw (&dc);
					((CLine*) m_lineArray[i])->DrawEndsW (&dc);
				}
			}
			else
			if(m_nTool == IDM_TOOL_ROTATE)
			{
				// draw white endpoints when line is not selected
				if(m_nSelectedLine != i)
				{
					((CLine*) m_lineArray[i])->Draw (&dc);
					((CLine*) m_lineArray[i])->DrawEndsWC (&dc);
				}
			}
			else
			if(m_nSelectedLine != i)
			{
				((CLine*) m_lineArray[i])->Draw (&dc);
			}
		}
    }

    nCount = m_triArray.GetSize ();

    if (nCount) 
	{
        for (int i=0; i<nCount; i++)
		{
			// draw black squares on the selected object
			if(m_nSelectedTriangle == i)
			{
				((CTri*) m_triArray[i])->Draw (&dc);
				if(m_nTool == IDM_TOOL_PICK)
					((CTri*) m_triArray[i])->DrawEndsB (&dc);
				else
					((CTri*) m_triArray[i])->DrawEndsBC (&dc);
			}

			if((m_nTool == IDM_TOOL_PICK) ||
				(m_nTool == IDM_TOOL_ERASE))
			{
				// draw white endpoints when triangle is not selected
				if(m_nSelectedTriangle != i)
				{
					((CTri*) m_triArray[i])->Draw (&dc);
					((CTri*) m_triArray[i])->DrawEndsW (&dc);
				}
			}
			else
			if(m_nTool == IDM_TOOL_ROTATE)
			{
				// draw white endpoints when triangle is not selected
				if(m_nSelectedTriangle != i)
				{
					((CTri*) m_triArray[i])->Draw (&dc);
					((CTri*) m_triArray[i])->DrawEndsWC (&dc);
				}
			}
			else
			if(m_nSelectedTriangle != i)
			{
				((CTri*) m_triArray[i])->Draw (&dc);
			}
		}
    }

    nCount = m_quadArray.GetSize ();

    if (nCount) 
	{
        for (int i=0; i<nCount; i++)
		{
			// draw black squares on the selected object
			if(m_nSelectedQuad == i)
			{
				((CQuad*) m_quadArray[i])->Draw (&dc);
				((CQuad*) m_quadArray[i])->DrawEndsB (&dc);
			}

			if((m_nTool == IDM_TOOL_PICK) ||
				(m_nTool == IDM_TOOL_ERASE))
			{
				// draw white endpoints when rect is not selected
				if(m_nSelectedQuad != i)
				{
					((CQuad*) m_quadArray[i])->Draw (&dc);
					((CQuad*) m_quadArray[i])->DrawEndsW (&dc);
				}
			}
			else
			if(m_nSelectedQuad != i)
			{
				((CQuad*) m_quadArray[i])->Draw (&dc);
			}
		}
    }

    nCount = m_circArray.GetSize ();

    if (nCount) 
	{
        for (int i=0; i<nCount; i++)
		{
			// draw black squares on the selected object
			if(m_nSelectedCirc == i)
			{
				((CCirc*) m_circArray[i])->Draw (&dc);
				((CCirc*) m_circArray[i])->DrawEndsB (&dc);
			}

			if(m_nTool == IDM_TOOL_PICK)
			{
				// draw white endpoints when circle is not selected
				if(m_nSelectedCirc != i)
				{
					((CCirc*) m_circArray[i])->Draw (&dc);
					((CCirc*) m_circArray[i])->DrawEndsW (&dc);
				}
				else
				if(m_nRubberCirc == i) // rubber circle is active?
				{
					InvertCirc(&dc, ((CCirc*) m_circArray[i])->m_ptFrom, 
						((CCirc*) m_circArray[i])->m_ptTo);
				}
			}
			else
			if(m_nTool == IDM_TOOL_ERASE)
			{
				// draw white endpoints when circle is not selected
				if(m_nSelectedCirc != i)
				{
					((CCirc*) m_circArray[i])->Draw (&dc);
					((CCirc*) m_circArray[i])->DrawEndsW (&dc);
				}
			}
			else
			if(m_nSelectedCirc != i)
			{
				((CCirc*) m_circArray[i])->Draw (&dc);
			}
		}
    }

	// Do not call CWnd::OnPaint() for painting messages
}

void CChildView::OnFileNew ()
{
    DeleteAllLines ();
    Invalidate ();
}

void CChildView::OnUpdateFileUI (CCmdUI* pCmdUI)
{
	if(m_lineArray.GetSize() > 0)
		pCmdUI->Enable (TRUE);
	else
	if(m_triArray.GetSize() > 0)
		pCmdUI->Enable (TRUE);
	else
	if(m_quadArray.GetSize() > 0)
		pCmdUI->Enable (TRUE);
	else
	if(m_circArray.GetSize() > 0)
		pCmdUI->Enable (TRUE);
	else
		pCmdUI->Enable (FALSE);
}

void CChildView::OnWidth (UINT nID)
{
    m_nWidth = nID;

	int nCount = m_lineArray.GetSize ();
	for (int i=nCount-1; i>=0; i--)
	{
		if(m_nSelectedLine == i)
		{
			((CLine*) m_lineArray[i])->m_nWidth = 
				nWidths[m_nWidth - IDM_WIDTH_VTHIN];
			Invalidate();
		}
	}

	nCount = m_triArray.GetSize ();
	for (i=nCount-1; i>=0; i--)
	{
		if(m_nSelectedTriangle == i)
		{
			((CTri*) m_triArray[i])->m_nWidth = 
				nWidths[m_nWidth - IDM_WIDTH_VTHIN];
			Invalidate();
		}
	}

	nCount = m_quadArray.GetSize ();
	for (i=nCount-1; i>=0; i--)
	{
		if(m_nSelectedQuad == i)
		{
			((CQuad*) m_quadArray[i])->m_nWidth = 
				nWidths[m_nWidth - IDM_WIDTH_VTHIN];
			Invalidate();
		}
	}

	nCount = m_circArray.GetSize ();
	for (i=nCount-1; i>=0; i--)
	{
		if(m_nSelectedCirc == i)
		{
			((CCirc*) m_circArray[i])->m_nWidth = 
				nWidths[m_nWidth - IDM_WIDTH_VTHIN];
			Invalidate();
		}
	}
}

void CChildView::OnColor (UINT nID)
{
    m_nColor = nID;

	int nCount = m_lineArray.GetSize ();
	for (int i=nCount-1; i>=0; i--)
	{
		if(m_nSelectedLine == i)
		{
			((CLine*) m_lineArray[i])->m_crColor = 
				crColors[m_nColor - IDM_COLOR_BLACK];
			Invalidate();
		}
	}

	nCount = m_triArray.GetSize ();
	for (i=nCount-1; i>=0; i--)
	{
		if(m_nSelectedTriangle == i)
		{
			((CTri*) m_triArray[i])->m_crColor = 
				crColors[m_nColor - IDM_COLOR_BLACK];
			Invalidate();
		}
	}

	nCount = m_quadArray.GetSize ();
	for (i=nCount-1; i>=0; i--)
	{
		if(m_nSelectedQuad == i)
		{
			((CQuad*) m_quadArray[i])->m_crColor = 
				crColors[m_nColor - IDM_COLOR_BLACK];
			Invalidate();
		}
	}

	nCount = m_circArray.GetSize ();
	for (i=nCount-1; i>=0; i--)
	{
		if(m_nSelectedCirc == i)
		{
			((CCirc*) m_circArray[i])->m_crColor = 
				crColors[m_nColor - IDM_COLOR_BLACK];
			Invalidate();
		}
	}
}

void CChildView::OnStyle (UINT nID)
{
    m_nStyle = nID;

	int nCount = m_lineArray.GetSize ();
	for (int i=nCount-1; i>=0; i--)
	{
		if(m_nSelectedLine == i)
		{
			((CLine*) m_lineArray[i])->m_nStyle = 
				nStyles[m_nStyle - IDM_STYLE_SOLID];
			Invalidate();
		}
	}

	nCount = m_triArray.GetSize ();
	for (i=nCount-1; i>=0; i--)
	{
		if(m_nSelectedTriangle == i)
		{
			((CTri*) m_triArray[i])->m_nStyle = 
				nStyles[m_nStyle - IDM_STYLE_SOLID];
			Invalidate();
		}
	}

	nCount = m_quadArray.GetSize ();
	for (i=nCount-1; i>=0; i--)
	{
		if(m_nSelectedQuad == i)
		{
			((CQuad*) m_quadArray[i])->m_nStyle = 
				nStyles[m_nStyle - IDM_STYLE_SOLID];
			Invalidate();
		}
	}

	nCount = m_circArray.GetSize ();
	for (i=nCount-1; i>=0; i--)
	{
		if(m_nSelectedCirc == i)
		{
			((CCirc*) m_circArray[i])->m_nStyle = 
				nStyles[m_nStyle - IDM_STYLE_SOLID];
			Invalidate();
		}
	}
}

void CChildView::OnTool (UINT nID)
{
	m_nTool = nID;

	if((m_nTool != IDM_TOOL_PICK) && 
		(m_nTool != IDM_TOOL_ROTATE))
	{
		m_nSelectedLine = -1;
		m_nSelectedTriangle = -1;
		m_nSelectedQuad = -1;
		m_nSelectedCirc = -1;
	}

	m_nRubberLine = -1;
	m_nRubberTriangle = -1;
	m_nRubberQuad = -1;
	m_nRubberCirc = -1;

	Invalidate();
}

//void CChildView::OnView (UINT nID)
//{
//}

void CChildView::OnUpdateWidthUI (CCmdUI* pCmdUI)
{
    pCmdUI->SetCheck (pCmdUI->m_nID == m_nWidth);
}

void CChildView::OnUpdateColorUI (CCmdUI* pCmdUI)
{
    pCmdUI->SetCheck (pCmdUI->m_nID == m_nColor);
}

void CChildView::OnUpdateStyleUI (CCmdUI* pCmdUI)
{
    pCmdUI->SetCheck (pCmdUI->m_nID == m_nStyle);
}

void CChildView::OnUpdateToolUI (CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck (pCmdUI->m_nID == m_nTool);
}

void CChildView::OnLButtonDown (UINT nFlags, CPoint point)
{
    CClientDC dc (this);

	int i, nCount;
	switch(m_nTool)
	{
	case IDM_TOOL_PICK:
		// any selected line?
		nCount = m_lineArray.GetSize ();
		for (i=nCount-1; i>=0; i--)
		{
			// get rotated point
			CPoint a, b;
			((CLine*) m_lineArray[i])->RotateLine(a, b);

			if((point.x >= a.x-5) && (point.x <= a.x+5) &&
				(point.y >= a.y-5) && (point.y <= a.y+5))
			{
				m_nSelectedLine = m_nSelectedTriangle = m_nSelectedQuad = m_nSelectedCirc = -1;
				m_nRubberLine = m_nSelectedLine = i;
				movingPt1 = TRUE;
				movingPt2 = FALSE;
				movingPt3 = FALSE;
				
				Invalidate();
				m_ptTo = b;
				m_ptFrom = point;
				((CLine*) m_lineArray[i])->m_ptFrom = point;
				((CLine*) m_lineArray[i])->angle = 0; // zero angle
				
				SetCapture();
				break;
			}
			else
			if((point.x >= b.x-5) && (point.x <= b.x+5) &&
				(point.y >= b.y-5) && (point.y <= b.y+5))
			{
				m_nSelectedLine = m_nSelectedTriangle = m_nSelectedQuad = m_nSelectedCirc = -1;				
				m_nRubberLine = m_nSelectedLine = i;
				movingPt2 = TRUE;
				movingPt1 = FALSE;
				movingPt3 = FALSE;

				Invalidate();
				m_ptFrom = a;
				m_ptTo = point;
				((CLine*) m_lineArray[i])->m_ptTo = point;
				((CLine*) m_lineArray[i])->angle = 0; // zero angle
				
				SetCapture();
				break;
			}
			else
			if(PointOnLine(a, b, 
				(float)((CLine*)m_lineArray[i])->m_nWidth, 
				point))
			{
				m_nSelectedLine = m_nSelectedTriangle = m_nSelectedQuad = m_nSelectedCirc = -1;				
				m_nRubberLine = m_nSelectedLine = i;

				Invalidate();
				
				m_ptFrom = a;
				m_ptTo = b;
				((CLine*) m_lineArray[i])->angle = 0; // zero angle
				click = point;
				
				SetCapture();

				break;
			}
		}
		if(m_nRubberLine != -1)
			break;

		// any selected triangle?
		nCount = m_triArray.GetSize ();
		for (i=nCount-1; i>=0; i--)
		{
			// get rotated points
			CPoint a, b, c;
			((CTri*) m_triArray[i])->RotateTri(a, b, c);

			if((point.x >= a.x-5) && (point.x <= a.x+5) &&
				(point.y >= a.y-5) && (point.y <= a.y+5))
			{
				m_nSelectedLine = m_nSelectedTriangle = m_nSelectedQuad = m_nSelectedCirc = -1;				
				m_nRubberTriangle = m_nSelectedTriangle = i;
				movingPt1 = TRUE;
				movingPt2 = FALSE;
				movingPt3 = FALSE;

				Invalidate();
				m_ptTri[1] = b;
				m_ptTri[2] = c;
				m_ptTri[0] = point;
				((CTri*) m_triArray[i])->m_pt[0] = point;
				((CTri*) m_triArray[i])->angle = 0; // zero angle
				SetCapture();
				break;
			}
			else
			if((point.x >= b.x-5) && (point.x <= b.x+5) &&
				(point.y >= b.y-5) && (point.y <= b.y+5))
			{
				m_nSelectedLine = m_nSelectedTriangle = m_nSelectedQuad = m_nSelectedCirc = -1;				
				m_nRubberTriangle = m_nSelectedTriangle = i;
				movingPt2 = TRUE;
				movingPt1 = FALSE;
				movingPt3 = FALSE;

				Invalidate();
				m_ptTri[1] = point;
				m_ptTri[0] = a;
				m_ptTri[2] = c;
				((CTri*) m_triArray[i])->m_pt[1] = point;
				((CTri*) m_triArray[i])->angle = 0; // zero angle
				SetCapture();
				break;
			}
			else
			if((point.x >= c.x-5) &&
				(point.x <= c.x+5) &&
				(point.y >= c.y-5) &&
				(point.y <= c.y+5))
			{
				m_nSelectedLine = m_nSelectedTriangle = m_nSelectedQuad = m_nSelectedCirc = -1;				
				m_nRubberTriangle = m_nSelectedTriangle = i;
				movingPt3 = TRUE;
				movingPt1 = FALSE;
				movingPt2 = FALSE;

				Invalidate();
				m_ptTri[2] = point;
				m_ptTri[0] = a;
				m_ptTri[1] = b;
				((CTri*) m_triArray[i])->m_pt[2] = point;
				((CTri*) m_triArray[i])->angle = 0; // zero angle
				SetCapture();
				break;
			}
			else
			if(PointOnLine(a, b, (float)((CTri*)m_triArray[i])->m_nWidth, point) ||
			   PointOnLine(b, c, (float)((CTri*)m_triArray[i])->m_nWidth, point) ||
			   PointOnLine(c, a, (float)((CTri*)m_triArray[i])->m_nWidth, point))
			{
				m_nSelectedLine = m_nSelectedTriangle = m_nSelectedQuad = m_nSelectedCirc = -1;
				m_nRubberTriangle = m_nSelectedTriangle = i;

				Invalidate();
				
				m_ptTri[0] = a;
				m_ptTri[1] = b;
				m_ptTri[2] = c;
				((CTri*) m_triArray[i])->angle = 0; // zero angle
				click = point;
				
				SetCapture();

				break;
			}
		}
		if(m_nRubberTriangle != -1)
			break;

		// any selected rectangle?
		nCount = m_quadArray.GetSize ();
		for (i=nCount-1; i>=0; i--)
		{
			// get rotated points
			CPoint a = ((CQuad*) m_quadArray[i])->m_ptFrom, 
				b = ((CQuad*) m_quadArray[i])->m_ptTo,
				c = CPoint(((CQuad*) m_quadArray[i])->m_ptTo.x,((CQuad*) m_quadArray[i])->m_ptFrom.y),
				d = CPoint(((CQuad*) m_quadArray[i])->m_ptFrom.x,((CQuad*) m_quadArray[i])->m_ptTo.y);

			if((point.x >= a.x-5) && (point.x <= a.x+5) &&
				(point.y >= a.y-5) && (point.y <= a.y+5))
			{
				m_nSelectedLine = m_nSelectedTriangle = m_nSelectedQuad = m_nSelectedCirc = -1;
				m_nRubberQuad = m_nSelectedQuad = i;
				movingPt1 = TRUE;
				movingPt2 = FALSE;
				movingPt3 = FALSE;
				
				Invalidate();
				m_ptTo = b;
				m_ptFrom = point;
				((CQuad*) m_quadArray[i])->m_ptFrom = point;
				SetCapture();
				break;
			}
			else
			if((point.x >= b.x-5) && (point.x <= b.x+5) &&
				(point.y >= b.y-5) && (point.y <= b.y+5))
			{
				m_nSelectedLine = m_nSelectedTriangle = m_nSelectedQuad = m_nSelectedCirc = -1;
				m_nRubberQuad = m_nSelectedQuad = i;
				movingPt2 = TRUE;
				movingPt1 = FALSE;
				movingPt3 = FALSE;

				Invalidate();
				m_ptFrom = a;
				m_ptTo = point;
				((CQuad*) m_quadArray[i])->m_ptTo = point;
				SetCapture();
				break;
			}
			else
			if(PointOnLine(a, c, (float)((CQuad*)m_quadArray[i])->m_nWidth, point) ||
			   PointOnLine(c, b, (float)((CQuad*)m_quadArray[i])->m_nWidth, point) ||	
			   PointOnLine(b, d, (float)((CQuad*)m_quadArray[i])->m_nWidth, point) ||
			   PointOnLine(d, a, (float)((CQuad*)m_quadArray[i])->m_nWidth, point))
			{
				m_nSelectedLine = m_nSelectedTriangle = m_nSelectedQuad = m_nSelectedCirc = -1;
				m_nRubberQuad = m_nSelectedQuad = i;

				Invalidate();
				
				m_ptFrom = a;
				m_ptTo = b;
				click = point;
				SetCapture();

				break;
			}
		}
		if(m_nRubberQuad != -1)
			break;

		// any selected circle?
		nCount = m_circArray.GetSize ();
		for (i=nCount-1; i>=0; i--)
		{
			if((point.x >= ((CCirc*) m_circArray[i])->m_ptFrom.x-5) &&
				(point.x <= ((CCirc*) m_circArray[i])->m_ptFrom.x+5) &&
				(point.y >= ((CCirc*) m_circArray[i])->m_ptFrom.y-5) &&
				(point.y <= ((CCirc*) m_circArray[i])->m_ptFrom.y+5))
			{
				m_nSelectedLine = m_nSelectedTriangle = m_nSelectedQuad = m_nSelectedCirc = -1;
				m_nRubberCirc = m_nSelectedCirc = i;
				movingPt1 = TRUE;
				movingPt2 = FALSE;
				movingPt3 = FALSE;
				
				Invalidate();
				m_ptTo = ((CCirc*) m_circArray[i])->m_ptTo;
				m_ptFrom.x = m_ptTo.x + (point.x - m_ptTo.x);
				m_ptFrom.y = m_ptTo.y + (point.x - m_ptTo.x);
				((CCirc*) m_circArray[i])->m_ptFrom = m_ptFrom;
				
				SetCapture();
				break;
			}
			else
			if((point.x >= ((CCirc*) m_circArray[i])->m_ptTo.x-5) &&
				(point.x <= ((CCirc*) m_circArray[i])->m_ptTo.x+5) &&
				(point.y >= ((CCirc*) m_circArray[i])->m_ptTo.y-5) &&
				(point.y <= ((CCirc*) m_circArray[i])->m_ptTo.y+5))
			{
				m_nSelectedLine = m_nSelectedTriangle = m_nSelectedQuad = m_nSelectedCirc = -1;
				m_nRubberCirc = m_nSelectedCirc = i;
				movingPt2 = TRUE;
				movingPt1 = FALSE;
				movingPt3 = FALSE;

				Invalidate();
				m_ptFrom = ((CCirc*) m_circArray[i])->m_ptFrom;
				m_ptTo.x = m_ptFrom.x + (point.x - m_ptFrom.x);
				m_ptTo.y = m_ptFrom.y + (point.x - m_ptFrom.x);
				((CCirc*) m_circArray[i])->m_ptTo = m_ptTo;
				
				SetCapture();
				break;
			}
			else
			if(PointOnCircle(((CCirc*)m_circArray[i])->m_ptFrom, ((CCirc*)m_circArray[i])->m_ptTo, (float)((CCirc*)m_circArray[i])->m_nWidth, point))
			{
				m_nSelectedLine = m_nSelectedTriangle = m_nSelectedQuad = m_nSelectedCirc = -1;
				m_nRubberCirc = m_nSelectedCirc = i;

				Invalidate();
				
				m_ptFrom = ((CCirc*) m_circArray[i])->m_ptFrom;
				m_ptTo = ((CCirc*) m_circArray[i])->m_ptTo;
				click = point;
				
				SetCapture();

				break;
			}
		}

		break;
	case IDM_TOOL_ERASE:
		// line was erased?
		nCount = m_lineArray.GetSize ();
		for (i=0; i<nCount; i++)
		{
			// get rotated point
			CPoint a, b;
			((CLine*) m_lineArray[i])->RotateLine(a, b);

			if(PointOnLine(a, b, 
				(float)((CLine*)m_lineArray[i])->m_nWidth, 
				point))
			{
				m_lineArray.RemoveAt(i);

				Invalidate();
				break;
			}
		}
		// triangle was erased?
		nCount = m_triArray.GetSize ();
		for (i=0; i<nCount; i++)
		{
			// get rotated point
			CPoint a, b, c;
			((CTri*) m_triArray[i])->RotateTri(a, b, c);


			if(PointOnLine(a, b, (float)((CTri*)m_triArray[i])->m_nWidth, point) ||
			   PointOnLine(b, c, (float)((CTri*)m_triArray[i])->m_nWidth, point) ||
			   PointOnLine(c, a, (float)((CTri*)m_triArray[i])->m_nWidth, point))
			{
				m_triArray.RemoveAt(i);

				Invalidate();
				break;
			}
		}
		// rectangle was erased?
		nCount = m_quadArray.GetSize ();
		for (i=0; i<nCount; i++)
		{
			CPoint a = ((CQuad*) m_quadArray[i])->m_ptFrom, 
				b = ((CQuad*) m_quadArray[i])->m_ptTo,
				c = CPoint(((CQuad*) m_quadArray[i])->m_ptTo.x,((CQuad*) m_quadArray[i])->m_ptFrom.y),
				d = CPoint(((CQuad*) m_quadArray[i])->m_ptFrom.x,((CQuad*) m_quadArray[i])->m_ptTo.y);


			if(PointOnLine(a, c, (float)((CQuad*)m_quadArray[i])->m_nWidth, point) ||
			   PointOnLine(c, b, (float)((CQuad*)m_quadArray[i])->m_nWidth, point) ||	
			   PointOnLine(b, d, (float)((CQuad*)m_quadArray[i])->m_nWidth, point) ||
			   PointOnLine(d, a, (float)((CQuad*)m_quadArray[i])->m_nWidth, point))
			{
				m_quadArray.RemoveAt(i);

				Invalidate();
				break;
			}
		}
		// circle was erased?
		nCount = m_circArray.GetSize ();
		for (i=0; i<nCount; i++)
		{
			if(PointOnCircle(((CCirc*)m_circArray[i])->m_ptFrom, 
				((CCirc*)m_circArray[i])->m_ptTo, 
				(float)((CCirc*)m_circArray[i])->m_nWidth, point))
			{
				m_circArray.RemoveAt(i);

				Invalidate();
				break;
			}
		}
		break;
	case IDM_TOOL_ROTATE:
		// rotating line?
		nCount = m_lineArray.GetSize ();
		for (i=nCount-1; i>=0; i--)
		{
			// get rotated point
			CPoint a, b;
			((CLine*) m_lineArray[i])->RotateLine(a, b);

			if(((point.x >= a.x-5) && (point.x <= a.x+5) &&
				(point.y >= a.y-5) && (point.y <= a.y+5)) ||
				((point.x >= b.x-5) && (point.x <= b.x+5) &&
				(point.y >= b.y-5) && (point.y <= b.y+5)))
			{
				m_nSelectedLine = m_nSelectedTriangle = m_nSelectedQuad = m_nSelectedCirc = -1;
				m_nRubberLine = m_nSelectedLine = i;

				rotating = TRUE;

				// calculate centre of rotation
				rotcenter.x = (a.x + b.x)/2;
				rotcenter.y = (a.y + b.y)/2;

				// calculate vector from COR to mouseclick
				vecstart = point - rotcenter;
				
				Invalidate();
				((CLine*) m_lineArray[i])->angle = 0; // zero angle
				
				SetCapture();
				break;
			}
		}
		if(m_nRubberLine != -1)
			break;

		// rotating triangle?
		nCount = m_triArray.GetSize ();
		for (i=nCount-1; i>=0; i--)
		{
			// get rotated point
			CPoint a, b, c;
			((CTri*) m_triArray[i])->RotateTri(a, b, c);

			if(((point.x >= a.x-5) && (point.x <= a.x+5) &&
				(point.y >= a.y-5) && (point.y <= a.y+5)) ||
				((point.x >= b.x-5) && (point.x <= b.x+5) &&
				(point.y >= b.y-5) && (point.y <= b.y+5)) ||
				((point.x >= c.x-5) && (point.x <= c.x+5) &&
				(point.y >= c.y-5) && (point.y <= c.y+5)))
			{
				m_nSelectedLine = m_nSelectedTriangle = m_nSelectedQuad = m_nSelectedCirc = -1;
				m_nRubberTriangle = m_nSelectedTriangle = i;

				rotating = TRUE;

				// calculate centre of rotation
				rotcenter.x = (a.x + b.x + c.x)/3;
				rotcenter.y = (a.y + b.y + c.y)/3;

				// calculate vector from COR to mouseclick
				vecstart = point - rotcenter;
				
				Invalidate();
				((CTri*) m_triArray[i])->angle = 0; // zero angle
				
				SetCapture();
				break;
			}
		}
		if(m_nRubberTriangle != -1)
			break;
		break;
	case IDM_TOOL_LINE:
		if(!lin_cur) 
			m_ptFrom = point;
		m_ptTo = point;
		lin_cur++;
		SetCapture ();
		break;
	case IDM_TOOL_TRI:
		m_ptTri[tri_cur] = point;
		tri_cur++;
		m_ptTri[tri_cur] = point;
		SetCapture();
		break;
	case IDM_TOOL_RECT:
		if(!quad_cur) 
			m_ptFrom = point;
		m_ptTo = point;
		quad_cur++;
		SetCapture ();
		break;
	case IDM_TOOL_CIRC:
		if(!circ_cur) 
			m_ptFrom = point;
		m_ptTo.x = m_ptFrom.x + (point.x - m_ptFrom.x);
		m_ptTo.y = m_ptFrom.y + (point.x - m_ptFrom.x);
		circ_cur++;
		SetCapture ();
		break;
	}
}

void CChildView::OnMouseMove (UINT nFlags, CPoint point)
{
    if (GetCapture () == this) 
	{
        CClientDC dc (this);

		if(m_nTool == IDM_TOOL_PICK)
		{
			// moving line?
			if(m_nRubberLine != -1)
			{
				if(movingPt1) // moving an endpoint
				{
					InvertLine (&dc, m_ptFrom, m_ptTo);
					InvertLine (&dc, point, m_ptTo);

					m_ptFrom = point;
				}
				else
				if(movingPt2)
				{
					InvertLine (&dc, m_ptFrom, m_ptTo);
					InvertLine (&dc, m_ptFrom, point);

					m_ptTo = point;
				}
				else 
				{
					InvertLine (&dc, m_ptFrom, m_ptTo);

					m_ptTo.x -= (click.x - point.x);
					m_ptTo.y -= (click.y - point.y);
					m_ptFrom.x -= (click.x - point.x);
					m_ptFrom.y -= (click.y - point.y);

					click = point;

					InvertLine (&dc, m_ptFrom, m_ptTo);
				}
			}


			// moving a triangle?
			if(m_nRubberTriangle != -1)
			{
				if(movingPt1) // moving pt1
				{
					InvertLine (&dc, m_ptTri[0], m_ptTri[1]);
					InvertLine (&dc, point, m_ptTri[1]);

					InvertLine (&dc, m_ptTri[0], m_ptTri[2]);
					InvertLine (&dc, point, m_ptTri[2]);

					m_ptTri[0] = point;
				}
				else
				if(movingPt2) // moving pt2
				{
					InvertLine (&dc, m_ptTri[0], m_ptTri[1]);
					InvertLine (&dc, m_ptTri[0], point);
				
					InvertLine (&dc, m_ptTri[1], m_ptTri[2]);
					InvertLine (&dc, point, m_ptTri[2]);

					m_ptTri[1] = point;
				}
				else
				if(movingPt3) // moving pt3
				{
					InvertLine (&dc, m_ptTri[1], m_ptTri[2]);
					InvertLine (&dc, m_ptTri[1], point);
				
					InvertLine (&dc, m_ptTri[0], m_ptTri[2]);
					InvertLine (&dc, m_ptTri[0], point);

					m_ptTri[2] = point;
				}
				else 
				{
					InvertLine (&dc, m_ptTri[0], m_ptTri[1]);
					InvertLine (&dc, m_ptTri[1], m_ptTri[2]);
					InvertLine (&dc, m_ptTri[2], m_ptTri[0]);

					m_ptTri[0].x -= (click.x - point.x);
					m_ptTri[0].y -= (click.y - point.y);
					m_ptTri[1].x -= (click.x - point.x);
					m_ptTri[1].y -= (click.y - point.y);
					m_ptTri[2].x -= (click.x - point.x);
					m_ptTri[2].y -= (click.y - point.y);

					click = point;

					InvertLine (&dc, m_ptTri[0], m_ptTri[1]);
					InvertLine (&dc, m_ptTri[1], m_ptTri[2]);
					InvertLine (&dc, m_ptTri[2], m_ptTri[0]);
				}
			}

			// moving rectangle?
			if(m_nRubberQuad != -1)
			{
				if(movingPt1)
				{
					InvertQuad (&dc, m_ptFrom, m_ptTo);
					InvertQuad (&dc, point, m_ptTo);

					m_ptFrom = point;
				}
				else
				if(movingPt2)
				{
					InvertQuad (&dc, m_ptFrom, m_ptTo);
					InvertQuad (&dc, m_ptFrom, point);

					m_ptTo = point;
				}
				else
				{
					InvertQuad (&dc, m_ptFrom, m_ptTo);

					m_ptTo.x -= (click.x - point.x);
					m_ptTo.y -= (click.y - point.y);
					m_ptFrom.x -= (click.x - point.x);
					m_ptFrom.y -= (click.y - point.y);

					click = point;

					InvertQuad (&dc, m_ptFrom, m_ptTo);
				}
			}

			// Moving a elipse?
			if(m_nRubberCirc != -1)
			{
				if(movingPt1)
				{
					InvertCirc (&dc, m_ptFrom, m_ptTo);

					m_ptFrom.x = m_ptTo.x + (point.x - m_ptTo.x);
					m_ptFrom.y = m_ptTo.y + (point.x - m_ptTo.x);

					InvertCirc (&dc, m_ptFrom, m_ptTo);
				}
				else
				if(movingPt2)
				{
					InvertCirc (&dc, m_ptFrom, m_ptTo);

					m_ptTo.x = m_ptFrom.x + (point.x - m_ptFrom.x);
					m_ptTo.y = m_ptFrom.y + (point.x - m_ptFrom.x);

					InvertCirc (&dc, m_ptFrom, m_ptTo);
				}
				else
				{
					InvertCirc (&dc, m_ptFrom, m_ptTo);

					m_ptTo.x -= (click.x - point.x);
					m_ptTo.y -= (click.y - point.y);
					m_ptFrom.x -= (click.x - point.x);
					m_ptFrom.y -= (click.y - point.y);

					click = point;

					InvertCirc (&dc, m_ptFrom, m_ptTo);
				}
			}
		}
		else if(m_nTool == IDM_TOOL_ROTATE)
		{
			// rotating line?
			if(m_nRubberLine != -1)
			{
				if(rotating)
				{
					// vector from COR to current cursor position
					CPoint vecend = point - rotcenter;

					// normalize vecstart
					double leng = sqrt(vecstart.x*vecstart.x + vecstart.y*vecstart.y);
					double vecstartx = vecstart.x / leng;
					double vecstarty = vecstart.y / leng;

					// normalize vecend
					leng = sqrt(vecend.x*vecend.x + vecend.y*vecend.y);
					double vecendx = vecend.x / leng;
					double vecendy = vecend.y / leng;

					CPoint a,b;
					((CLine*) m_lineArray[m_nSelectedLine])->RotateLine(a, b);
					InvertLine (&dc, a, b);

					// now calculate angle between vecend and vecstart
					float cosa = vecendx*vecstartx + vecendy*vecstarty;
					float dist = DistPointOnLine(CPoint(0,0), vecstart, vecend);
					if(dist < 0) cosa = -cosa;
					float angle = acos(cosa);
					((CLine*) m_lineArray[m_nSelectedLine])->angle = angle;

					((CLine*) m_lineArray[m_nSelectedLine])->RotateLine(a, b);
					InvertLine (&dc, a, b);
				}
			}

			// rotating triangle?
			if(m_nRubberTriangle != -1)
			{
				if(rotating)
				{
					// vector from COR to current cursor position
					CPoint vecend = point - rotcenter;

					// normalize vecstart
					double leng = sqrt(vecstart.x*vecstart.x + vecstart.y*vecstart.y);
					double vecstartx = vecstart.x / leng;
					double vecstarty = vecstart.y / leng;

					// normalize vecend
					leng = sqrt(vecend.x*vecend.x + vecend.y*vecend.y);
					double vecendx = vecend.x / leng;
					double vecendy = vecend.y / leng;

					CPoint a,b,c;
					((CTri*) m_triArray[m_nSelectedTriangle])->RotateTri(a, b, c);
					InvertLine (&dc, a, b);
					InvertLine (&dc, b, c);
					InvertLine (&dc, c, a);

					// now calculate angle between vecend and vecstart
					float cosa = vecendx*vecstartx + vecendy*vecstarty;
					float angle = acos(cosa);
					((CTri*) m_triArray[m_nSelectedTriangle])->angle = angle;

					((CTri*) m_triArray[m_nSelectedTriangle])->RotateTri(a, b, c);
					InvertLine (&dc, a, b);
					InvertLine (&dc, b, c);
					InvertLine (&dc, c, a);
				}
			}
		}
		else if(m_nTool == IDM_TOOL_LINE) // creating line
		{
			InvertLine (&dc, m_ptFrom, m_ptTo);
			InvertLine (&dc, m_ptFrom, point);
			m_ptTo = point;
		}
		else if(m_nTool == IDM_TOOL_TRI) // creating triangle
		{
			// Triangulo
			InvertLine (&dc, m_ptTri[tri_cur-1], m_ptTri[tri_cur]);
			InvertLine (&dc, m_ptTri[tri_cur-1], point);
			m_ptTri[tri_cur] = point;
		}
		else if(m_nTool == IDM_TOOL_RECT) // creating rectangle
		{
			InvertQuad (&dc, m_ptFrom, m_ptTo);
			InvertQuad (&dc, m_ptFrom, point);
			m_ptTo = point;
		}
		else if(m_nTool == IDM_TOOL_CIRC) // creating circle
		{
			InvertCirc (&dc, m_ptFrom, m_ptTo);
			m_ptTo.x = m_ptFrom.x + (point.x - m_ptFrom.x);
			m_ptTo.y = m_ptFrom.y + (point.x - m_ptFrom.x);
			InvertCirc (&dc, m_ptFrom, m_ptTo);
		}
    }
}

void CChildView::OnLButtonUp (UINT nFlags, CPoint point)
{
	if(m_nTool == IDM_TOOL_PICK)
	{
		if (GetCapture () == this) {
			ReleaseCapture ();

			CClientDC dc (this);

			// Lines
			if(m_nRubberLine != -1)
			{
				((CLine*) m_lineArray[m_nSelectedLine])->m_ptTo = m_ptTo;
				((CLine*) m_lineArray[m_nSelectedLine])->m_ptFrom = m_ptFrom;

				movingPt1 = movingPt2 = movingPt3 = FALSE;

				m_nRubberLine = -1;

				Invalidate();
				return;
			}

			// triangles
			if(m_nRubberTriangle != -1)
			{
				((CTri*) m_triArray[m_nSelectedTriangle])->m_pt[0] = m_ptTri[0];
				((CTri*) m_triArray[m_nSelectedTriangle])->m_pt[1] = m_ptTri[1];
				((CTri*) m_triArray[m_nSelectedTriangle])->m_pt[2] = m_ptTri[2];

				movingPt1 = movingPt2 = movingPt3 = FALSE;

				m_nRubberTriangle = -1;

				Invalidate();
				return;
			}

			// rectangles
			if(m_nRubberQuad != -1)
			{
				((CQuad*) m_quadArray[m_nSelectedQuad])->m_ptTo = m_ptTo;
				((CQuad*) m_quadArray[m_nSelectedQuad])->m_ptFrom = m_ptFrom;

				movingPt1 = movingPt2 = movingPt3 = FALSE;

				m_nRubberQuad = -1;

				Invalidate();
				return;
			}

			// circles
			if(m_nRubberCirc != -1)
			{
				((CCirc*) m_circArray[m_nSelectedCirc])->m_ptTo = m_ptTo;
				((CCirc*) m_circArray[m_nSelectedCirc])->m_ptFrom = m_ptFrom;

				movingPt1 = movingPt2 = movingPt3 = FALSE;

				m_nRubberCirc = -1;

				Invalidate();
				return;
			}
		}
	}

	if(m_nTool == IDM_TOOL_ROTATE)
	{
		if (GetCapture () == this) 
		{
			ReleaseCapture ();

			CClientDC dc (this);

			// lines
			if(m_nRubberLine != -1)
			{
				rotating = FALSE;

				((CLine*) m_lineArray[m_nSelectedLine])->RotateDef();
				((CLine*) m_lineArray[m_nSelectedLine])->angle = 0;

				m_nRubberLine = -1;

				Invalidate();
				return;
			}

			// triangles
			if(m_nRubberTriangle != -1)
			{
				rotating = FALSE;

				((CTri*) m_triArray[m_nSelectedTriangle])->RotateDef();
				((CTri*) m_triArray[m_nSelectedTriangle])->angle = 0;

				m_nRubberTriangle = -1;

				Invalidate();
				return;
			}
		}
	}

	if(m_nTool == IDM_TOOL_LINE)
	{
	if(lin_cur < 2)
		return;

    if (GetCapture () == this) {
        ReleaseCapture ();

        CClientDC dc (this);

		// add line
        CLine* pLine = NULL;

        try {
            pLine = new CLine (m_ptFrom, m_ptTo,
                nWidths[m_nWidth - IDM_WIDTH_VTHIN],
				nStyles[m_nStyle - IDM_STYLE_SOLID],
                crColors[m_nColor - IDM_COLOR_BLACK]);

            m_lineArray.Add (pLine);

			Invalidate();

			lin_cur = 0;
        }
        catch (CMemoryException* e) {
            MessageBox ("Out of memory. You must clear the " \
                "drawing area before adding more lines.", "Error",
                MB_ICONEXCLAMATION | MB_OK);

            if (pLine != NULL)
                delete pLine;
            e->Delete ();   
        }
    }
	}

	if(m_nTool == IDM_TOOL_TRI)
	{
	if(tri_cur < 3)
		return;

    if (GetCapture () == this) {
        ReleaseCapture ();

        CClientDC dc (this);
        
		// add triangle
        CTri* pTri = NULL;

        try {
			pTri = new CTri (m_ptTri[0], m_ptTri[1], m_ptTri[2],
				nWidths[m_nWidth - IDM_WIDTH_VTHIN],
				nStyles[m_nStyle - IDM_STYLE_SOLID],
				crColors[m_nColor - IDM_COLOR_BLACK]);

			m_triArray.Add (pTri);

			Invalidate();
			
			tri_cur = 0;
        }
        catch (CMemoryException* e) {
            MessageBox ("Out of memory. You must clear the " \
                "drawing area before adding more triangles.", "Error",
                MB_ICONEXCLAMATION | MB_OK);

            if (pTri != NULL)
                delete pTri;
            e->Delete ();   
        }       

    }
	}

	if(m_nTool == IDM_TOOL_RECT)
	{
	if(quad_cur < 2)
		return;

    if (GetCapture () == this) {
        ReleaseCapture ();

        CClientDC dc (this);
       

		// add rectangle
        CQuad* pQuad = NULL;

        try {
            pQuad = new CQuad (m_ptFrom, m_ptTo,
                nWidths[m_nWidth - IDM_WIDTH_VTHIN],
				nStyles[m_nStyle - IDM_STYLE_SOLID],
                crColors[m_nColor - IDM_COLOR_BLACK]);

            m_quadArray.Add (pQuad);
			Invalidate();

			quad_cur = 0;
        }
        catch (CMemoryException* e) {
            MessageBox ("Out of memory. You must clear the " \
                "drawing area before adding more rectangles.", "Error",
                MB_ICONEXCLAMATION | MB_OK);

            if (pQuad != NULL)
                delete pQuad;
            e->Delete ();   
        }
    }
	}

	if(m_nTool == IDM_TOOL_CIRC)
	{
	if(circ_cur < 2)
		return;

    if (GetCapture () == this) {
        ReleaseCapture ();

        CClientDC dc (this);
       
		// add circle
        CCirc* pCirc = NULL;

        try {
            pCirc = new CCirc (m_ptFrom, m_ptTo,
                nWidths[m_nWidth - IDM_WIDTH_VTHIN],
				nStyles[m_nStyle - IDM_STYLE_SOLID],
                crColors[m_nColor - IDM_COLOR_BLACK]);

            m_circArray.Add (pCirc);
			Invalidate();

			circ_cur = 0;
        }
        catch (CMemoryException* e) {
            MessageBox ("Out of memory. You must clear the " \
                "drawing area before adding more circles.", "Error",
                MB_ICONEXCLAMATION | MB_OK);

            if (pCirc != NULL)
                delete pCirc;
            e->Delete ();   
        }
    }
	}
}

void CChildView::OnMeasureItem (int nIDCtl, LPMEASUREITEMSTRUCT lpmis)
{
    lpmis->itemWidth = ::GetSystemMetrics (SM_CYMENU) * 4;
    lpmis->itemHeight = ::GetSystemMetrics (SM_CYMENU);
}

void CChildView::OnDrawItem (int nIDCtl, LPDRAWITEMSTRUCT lpdis)
{
    BITMAP bm;
    CBitmap bitmap;
    bitmap.LoadOEMBitmap (32760); // #define OBM_CHECK 32760
    bitmap.GetObject (sizeof (bm), &bm);

    CDC dc;
    dc.Attach (lpdis->hDC);

    CBrush* pBrush = new CBrush (::GetSysColor ((lpdis->itemState &
        ODS_SELECTED) ? COLOR_HIGHLIGHT : COLOR_MENU));
    dc.FrameRect (&(lpdis->rcItem), pBrush);
    delete pBrush;

    if (lpdis->itemState & ODS_CHECKED) {
        CDC dcMem;
        dcMem.CreateCompatibleDC (&dc);
        CBitmap* pOldBitmap = dcMem.SelectObject (&bitmap);

        dc.BitBlt (lpdis->rcItem.left + 4, lpdis->rcItem.top +
            (((lpdis->rcItem.bottom - lpdis->rcItem.top) -
            bm.bmHeight) / 2), bm.bmWidth, bm.bmHeight, &dcMem,
            0, 0, SRCCOPY);

        dcMem.SelectObject (pOldBitmap);
    }

    pBrush = new CBrush (crColors[lpdis->itemID - IDM_COLOR_BLACK]);
    CRect rect = lpdis->rcItem;
    rect.DeflateRect (6, 4);
    rect.left += bm.bmWidth;
    dc.FillRect (rect, pBrush);
    delete pBrush;

    dc.Detach ();
}

void CChildView::InvertLine (CDC* pDC, CPoint ptFrom, CPoint ptTo)
{
    int nOldMode = pDC->SetROP2 (R2_NOT);

    pDC->MoveTo (ptFrom);
    pDC->LineTo (ptTo);

    pDC->SetROP2 (nOldMode);
}

void CChildView::InvertQuad (CDC* pDC, CPoint ptFrom, CPoint ptTo)
{
    int nOldMode = pDC->SetROP2 (R2_NOT);

    pDC->MoveTo (ptFrom);
    pDC->LineTo (ptTo.x, ptFrom.y);

    pDC->MoveTo (ptTo.x, ptFrom.y);
    pDC->LineTo (ptTo);

    pDC->MoveTo (ptTo);
    pDC->LineTo (ptFrom.x, ptTo.y);

	pDC->MoveTo (ptFrom.x, ptTo.y);
	pDC->LineTo (ptFrom);

    pDC->SetROP2 (nOldMode);
}

void CChildView::InvertCirc (CDC* pDC, CPoint ptFrom, CPoint ptTo)
{
    int nOldMode = pDC->SetROP2 (R2_NOT);

    pDC->Ellipse(ptFrom.x, ptFrom.y, ptTo.x, ptTo.y);

    pDC->SetROP2 (nOldMode);
}

void CChildView::DeleteAllLines ()
{
    int nCount = m_lineArray.GetSize ();

    for (int i=0; i<nCount; i++)
        delete m_lineArray[i];

    m_lineArray.RemoveAll ();

    nCount = m_triArray.GetSize ();

    for (i=0; i<nCount; i++)
        delete m_triArray[i];

    m_triArray.RemoveAll ();

    nCount = m_quadArray.GetSize ();

    for (i=0; i<nCount; i++)
        delete m_quadArray[i];
    
	m_quadArray.RemoveAll ();

    nCount = m_circArray.GetSize ();

    for (i=0; i<nCount; i++)
        delete m_circArray[i];
    
	m_circArray.RemoveAll ();

}

//
// calcula a distancia do ponto tst  reta definida pelos pontos
// a,b com largura w
bool PointOnLine(CPoint a, CPoint b, float w, CPoint tst)
{
	float x1 = (float)a.x;
	float y1 = (float)a.y;
	float x2 = (float)b.x;
	float y2 = (float)b.y;

	float dy = y1 - y2;
	float dx = x2 - x1;
	float len = (float)sqrt((dy*dy)+(dx*dx));
	float A = dy/len;
	float B = dx/len;
	float C = -((dy*x1) + (dx*y1))/len;

	float dist = (A*tst.x) + (B*tst.y) + C; // Distancia do ponto  reta
	float Nx = tst.x - A*dist; // Ponto mais prximo na reta
	float Ny = tst.y - B*dist; // Ponto mais prximo na reta

	float width = w/2;

	if(width < 4) width = 4;

	float minx = -width/2, maxx = width/2,
		  miny = -width/2, maxy = width/2;

	if(x1 > x2) { maxx += x1; minx += x2; }
	else { maxx += x2; minx += x1; }
	if(y1 > y2) { maxy += y1; miny += y2; }
	else { maxy += y2; miny += y1; }

	if((fabs(dist) < width) &&
	   ((Nx > minx) && (Nx < maxx)) && 
	   ((Ny > miny) && (Ny < maxy)))
		return true;
	
	return false;
}

//
// Calcula a distancia do ponto tst  reta definida pelos pontos
// a,b com largura w
float DistPointOnLine(CPoint a, CPoint b, CPoint tst)
{
	float x1 = (float)a.x;
	float y1 = (float)a.y;
	float x2 = (float)b.x;
	float y2 = (float)b.y;

	float dy = y1 - y2;
	float dx = x2 - x1;
	float len = (float)sqrt((dy*dy)+(dx*dx));
	float A = dy/len;
	float B = dx/len;
	float C = -((dy*x1) + (dx*y1))/len;

	float dist = (A*tst.x) + (B*tst.y) + C; // Distancia do ponto  reta

	return dist;
}

//
// Calcula a distancia do ponto tst ao contorno do circulo
// definida pela caixa envoltria com coordenadas a,b 
// e largura w
bool PointOnCircle(CPoint a, CPoint b, float w, CPoint tst)
{
	float L = (float)a.x;
	float T = (float)a.y;
	float R = (float)b.x;
	float B = (float)b.y;

	float Xc = (L+R)/2;
	float Yc = (T+B)/2;
	
	// Ponto mais proximo do circulo unitario equivalente
	float dx = tst.x - Xc;
	float dy = tst.y - Yc;
	float radius = (B-T)/2;
	float dist = (float)sqrt((dx*dx)+(dy*dy)) - radius;

	float width = w/2;

	if(width < 4) width = 4;

	if(fabs(dist) < width)
		return true;
	
	return false;
}